#include "BallFrame.h"

BallFrame::BallFrame( NewtonWorld* world, int newtonMat ) : PhysicsObject()
{
	mWorld = world;


	// first, load the visual object for the Ball Frame.
	mObjNum = Helpers::getOpenObjNum();
	dbLoadObject( "..\\media\\models\\basic_frame.x", mObjNum );
	
	mWindowObj = Helpers::getOpenObjNum();
	dbLoadObject( "..\\media\\models\\frame_win.x", mWindowObj );
	dbSetObjectTransparency( mWindowObj, 1 );
	dbSetObjectFilter( mWindowObj, 0 );
	dbDisableObjectZDepth( mWindowObj );
	dbSetObjectLight( mWindowObj, 0 );


	// now create the various pieces of geometry for newton collision.
	NewtonCollision* colarray[12];
	float matrix[16];

	// left wall piece.
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(-5.5,0,0), matrix );
	colarray[0] = NewtonCreateBox( mWorld, 3.0f, 10.0f, 2.0f, matrix );
	NewtonConvexCollisionSetUserID( colarray[0], CP_FRAME );

	// right wall piece
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(5.5,0,0), matrix );
	colarray[1] = NewtonCreateBox( mWorld, 3.0f, 10.0f, 2.0f, matrix );
	NewtonConvexCollisionSetUserID( colarray[1], CP_FRAME );

	// bottom floor piece
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(0,-6,0), matrix );
	colarray[2] = NewtonCreateBox( mWorld, 14.0f, 2.0f, 2.0f, matrix );
	NewtonConvexCollisionSetUserID( colarray[2], CP_FRAME );

	// front wall
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(0,0,-0.775f), matrix );
	colarray[3] = NewtonCreateBox( mWorld, 8.0f, 10.0f, 0.45f, matrix );
	NewtonConvexCollisionSetUserID( colarray[3], CP_FRONT );

	// back wall
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(0,0,0.775f), matrix );
	colarray[4] = NewtonCreateBox( mWorld, 8.0f, 10.0f, 0.45f, matrix );
	NewtonConvexCollisionSetUserID( colarray[4], CP_BACK );

	// preview tube back wall
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(8.0f,-2.5f,0.775f), matrix );
	colarray[5] = NewtonCreateBox( mWorld, 2.0f, 15.0f, 0.45f, matrix );
	NewtonConvexCollisionSetUserID( colarray[5], CP_FRAME );

	// preview tube front wall
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(8.0f,-2.5f,-0.775f), matrix );
	colarray[6] = NewtonCreateBox( mWorld, 2.0f, 15.0f, 0.45f, matrix );
	NewtonConvexCollisionSetUserID( colarray[6], CP_FRAME );

	// preview tube left wall
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(7.225f,-2.5f,0), matrix );
	colarray[7] = NewtonCreateBox( mWorld, 0.45f, 15.0f, 1.1f, matrix );
	NewtonConvexCollisionSetUserID( colarray[7], CP_FRAME );

	// preview tube right wall
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(8.775f,-2.5f,0.775f), matrix );
	colarray[8] = NewtonCreateBox( mWorld, 0.45f, 15.0f, 1.1f, matrix );
	NewtonConvexCollisionSetUserID( colarray[8], CP_FRAME );

	// preview stopper
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(8.0f,-4.0f,0), matrix );
	colarray[9] = NewtonCreateBox( mWorld, 2.0f, 0.5f, 2.0f, matrix );
	NewtonConvexCollisionSetUserID( colarray[9], CP_PREVIEW );

	// shoot stopper
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(8.0,-5.5,0), matrix );
	colarray[10] = NewtonCreateBox( mWorld, 2.0f, 0.5f, 2.0f, matrix );
	NewtonConvexCollisionSetUserID( colarray[10], CP_SHOOT );

	// shoot trigger
	Helpers::makeFloatMatrix( dVector(0,0,0), dVector(8.0,-6.8,0), matrix );
	colarray[11] = NewtonCreateBox( mWorld, 2.0f, 0.5f, 2.0f, matrix );
	NewtonConvexCollisionSetUserID( colarray[11], CP_SHOOT_TRIGGER );




	// make the compound collision piece.
	NewtonCollision* col = NewtonCreateCompoundCollision( mWorld, 12, colarray );

	// make the rigid body.
	mBody = NewtonCreateBody( mWorld, col );

	// set the newton material.
	NewtonBodySetMaterialGroupID( mBody, newtonMat );

	// set the type!
	setType( BT_FRAME );

	// release the compound collision.
	NewtonReleaseCollision( mWorld, col );

	// setup the phyics
	_doPhysicsWiring();
	
}

BallFrame::~BallFrame()
{
	dbDeleteObject( mWindowObj );

}





