/*
	ConnectMore project

	by Walaber

	CLASS: BasicBall - basic class for all balls in the game.
*/

#ifndef _BASICBALL_INCLUDED_
#define _BASICBALL_INCLUDED_

#include "Prereqs.h"

#include "HelperFunctions.h"
#include "PhysicsObject.h"
#include <Newton.h>
#include <DarkSDK.h>

extern FILE* debugLog;

class BasicBall : public PhysicsObject
{
public:

	typedef std::map< int, BasicBall* > BallContactMap;
	typedef std::map< int, BasicBall* >::iterator BallContactMapIterator;

	// constructor.
	BasicBall();
	// full constructor that creates ball.
	BasicBall( NewtonWorld* world, ColorInfo color, dVector pos, int newtonMat, int objID );

	//destructor
	~BasicBall();

	// main initialization function.  must be called to create a ball.
	void init( NewtonWorld* world, ColorInfo color, dVector pos, int newtonMat, int objID );

	// get the color of this ball.
	ColorInfo getColorInfo() { return mColor; }

	// bool set/get functions.
	bool canPassFront() { return mCanPassFront; }
	void canPassFront( bool set ) { mCanPassFront = set; }

	bool canPassBack() { return mCanPassBack; }
	void canPassBack( bool set ) { mCanPassBack = set; }

	bool canPassPreview() { return mCanPassPreview; }
	void canPassPreview(bool set ) { mCanPassPreview = set; }

	bool canPassShoot() { return mCanPassShoot; }
	void canPassShoot( bool set ) { mCanPassShoot = true; }

	bool isPreviewBall() { return mPreviewBall; }
	void isPreviewBall( bool set ) { mPreviewBall = set; }


	// possible ball touching this one...
	int possibleTouch( BasicBall* ball );

	BallContactMap getContactMap() { return mBallsInContact; }


	// hack to clear maps
	static void _cdecl _BasicBallClearCallback( const NewtonBody* body );

	//special variable for user-shot balls.
	int mBallsHit;

private:

	// ball color!
	ColorInfo mColor;

	// bools that define collision behavior.
	bool mCanPassFront;
	bool mCanPassBack;
	bool mCanPassPreview;
	bool mCanPassShoot;
	bool mPreviewBall;

	// map of same-color balls in contact with this one.
	BallContactMap mBallsInContact;

	


};

#endif // _BASICBALL_INCLUDED_