/*
	ConnectMore 

	a DarkSDK game by Walaber (http://walaber.com)

	email: walaber@walaber.com

	for the TGC Puzzle Game Compo, July 2005

	this source has been made available to help those interested in using Newton with DarkDSK.
	feel free to use it in any way you see fit.

	The "PhysicsObject" class is a good starting point for any DarkSDK+Newton project.
	also the SoundManager is a quick and easy way to handle sounds with DarkSDK, feel free to 
	use it if you want.

	thanks,

		walaber
*/
#include "Main.h"

// ugly globals
int ballCount = 0;
FILE* debugLog = NULL;



WorldInfo::WorldInfo()
{
	mState = GS_MAIN_MENU;
	mWorld = NULL;
	mMatBall = -1;
	mMatFrame = -1;
	mRootBallObjNum = -1;
	mRootSmoke1ImgNum = mRootSmoke2ImgNum = -1;
	mTimeToNextBall = 0.0f;
	mCurrentSpout = 0;;
	mScore = 0;
	mComboCount = 0;
	mMaxCombo = 0;
	mKillAllLikeMe = NULL;
	mFadeSpriteImg = 0;
	mFadeSprite = 0;
}


// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
// main game function
// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
void DarkSDK( void )
{
	// -----------------------------------------------------
	// INITIALIZATION
	// setup Newton world.
	WorldInfo& info = WorldInfo::getSingleton();

#ifdef _DEBUG_LOG
	debugLog = fopen("debugLog.txt", "wt");
	fprintf( debugLog, "debugLog.txt created.\n");
#endif

	const float goalPhysicsFPS = 90.0f;
	const float physicsUpdate = (1.0f)/(goalPhysicsFPS);

	info.mWorld = NewtonCreate( NULL, NULL );

#ifdef _DEBUG_LOG
	fprintf( debugLog, "NewtonWorld created.\n");
#endif

	// set world size!
	dVector worldmin(-20.0f,-20.0f,-30.0f);
	dVector worldmax(20.0f,20.0f,10.0f);
	NewtonSetWorldSize( info.mWorld, &worldmin.m_x, &worldmax.m_x );

	// leave world callback
	NewtonSetBodyLeaveWorldEvent( info.mWorld, newtonLeaveWorld );

	// setup Newton materials.
	info.mMatBall = NewtonMaterialGetDefaultGroupID( info.mWorld );
	info.mMatFrame = NewtonMaterialCreateGroupID( info.mWorld );

	// setup the callbacks.
	CollisionData mColData;
	NewtonMaterialSetCollisionCallback( info.mWorld, info.mMatBall, info.mMatFrame, &mColData, newtonContactBegin, newtonContactProcess, newtonContactEnd );
	NewtonMaterialSetCollisionCallback( info.mWorld, info.mMatBall, info.mMatBall, &mColData, newtonContactBegin, newtonContactProcess, newtonContactEnd );

	mColData.mNextBall = NULL;
	mColData.mShootBall = NULL;

#ifdef _DEBUG_LOG
	fprintf( debugLog, "Newton materials created.\n");
#endif

	// -------------------------------
	// basic DSDK setup.
#ifndef _DEBUG
	dbSetWindowOff();
#endif
	dbSetDisplayMode( 1024, 768, 32 );
	dbSetWindowPosition( 100,100 );
	dbSyncOn();
	dbSyncRate( 0 );
	dbAutoCamOff();
	dbHideMouse();

	// mouse
	dbPositionMouse( 0, 0 );
	info.mMouseX = 0;
	info.mMouseY = 0;
	info.mMouseMoveX = 0;
	info.mMouseMoveY = 0;

	dbHideLight( 0 );

	dbMakeLight( 1 );
	dbSetDirectionalLight( 1, 0.0f, -1.0f, 0.3f );

	dbMakeLight( 2 );
	dbSetPointLight( 2, 0, 0, 0 );
	dbSetLightRange( 2, 5.0f );

	dbSetAmbientLight( 20 ); 

	// setup the colors
	setupColors();

	// root ball object, from which others are cloned.
	info.mRootBallObjNum = Helpers::getOpenObjNum();
	dbLoadObject( "..\\media\\models\\ball.x", info.mRootBallObjNum );
	dbHideObject( info.mRootBallObjNum );

	// root smoke particle images.
	info.mRootSmoke1ImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\smoke1.png", info.mRootSmoke1ImgNum );

	info.mRootSmoke2ImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\smoke2.png", info.mRootSmoke2ImgNum );

	// -------------------
	// load basic sounds
	SoundManager::getSingleton().addSound( std::string("shoot"), std::string("..\\media\\sounds\\SHOOT2.wav") );
	SoundManager::getSingleton().addSound( std::string("hit"), std::string("..\\media\\sounds\\HIT2.wav") );
	SoundManager::getSingleton().addSound( std::string("clear"), std::string("..\\media\\sounds\\CLEAR.wav") );
	SoundManager::getSingleton().addSound( std::string("bomb"), std::string("..\\media\\sounds\\BOMB.wav") );
	SoundManager::getSingleton().addSound( std::string("wild"), std::string("..\\media\\sounds\\WILD.wav") );

	SoundManager::getSingleton().addMusic( std::string("music"), std::string("..\\media\\sounds\\MENU_LOOP.mp3") );
	SoundManager::getSingleton().loopMusic(std::string("music"));


	// -----------
	// main camera
	dbPositionCamera( -0.7f, 0.0f, -14.1f ); 
	dbPointCamera( -0.84f, -0.2f, 0.0f );



	// sky sphere!
	SkySphere* mSkySphere = new SkySphere("..\\media\\models\\tube_piece.x", "..\\media\\images\\back2.png");

	// -----------
	// make the floor object.
	BallFrame* mBallFrame = new BallFrame( info.mWorld, info.mMatFrame );

	// -------------------
	//  crosshair object...
	UserCrosshair* mCrosshair = NULL;

	// overflow checker
	OverflowChecker* mOverflowChecker = NULL;

	// setup the spouts.
	setupSpouts();

	// initial game state
	info.mState = GS_LOAD_MENU;

	dbRandomize( dbTimer() );

	setupFader( dbRgb(255,255,255) );
	dbSetSpriteAlpha( info.mFadeSprite, 255 );


	// --------------------------------------------------------------
	ScoreBoard* mMainScoreboard = new ScoreBoard(10, 10, 0.36f, 0.84f, 0.1f,"..\\media\\images\\font.png", dVector(-8.2f, 4.2f,-3.9f), dVector(0.0f, -50.0f,0.0f));
	ScoreBoard* mScoreBoard = new ScoreBoard(10, 1, 0.8f, 1.6f, 0.1f, "..\\media\\images\\font.png", dVector(-4.0f,-5.8f,-2.5f), dVector(65.0f, 0.0f, 0.0f));
	


#ifdef _DEBUG_LOG
	fprintf( debugLog, "BallFrame created.\n");
#endif
	
	// -----------------------------------------------------
	// MAIN GAME LOOP
	int oldtime = dbTimer();
	float elapsedInSec = 0.0f;
    
	// -------------------------------------------------
	//	MAIN GAME LOOP
	// -------------------------------------------------
	while ( LoopSDK() )
	{
		// update mouse.
		info.mMouseMoveX = dbMouseMoveX();
		info.mMouseMoveY = dbMouseMoveY();
		info.mMouseX += info.mMouseMoveX;
		info.mMouseY += info.mMouseMoveY;
		if (info.mMouseX < 0) { info.mMouseX = 0; }
		if (info.mMouseY < 0) { info.mMouseY = 0; }
		if (info.mMouseX > dbScreenWidth()) { info.mMouseX = dbScreenWidth(); }
		if (info.mMouseY > dbScreenHeight()) { info.mMouseY = dbScreenHeight(); }

		// ------------------------------------------------------
		//	INIT MAIN MENU
		// ------------------------------------------------------
		if (info.mState == GS_LOAD_MENU)
		{
			// create the menu manager.
			info.mMenuMgr = new MenuManager();

			info.mMenuMgr->setScreenSize( dbScreenWidth(), dbScreenHeight() );
			info.mMenuMgr->addCursor( "..\\media\\images\\menu\\cursor.png");

			// create the logo.
			int xmid = (dbScreenWidth()/2);
			int ymid = (dbScreenHeight()/2);
			MenuPiece* title = new MenuPiece(MM_TITLE, xmid-256, ymid-250, "..\\media\\images\\menu\\title.png");

			// play button.
			MenuButton* play = new MenuButton(MM_PLAY_BUT, xmid-155, ymid-20, "..\\media\\images\\menu\\play1.png",
												"..\\media\\images\\menu\\play2.png", "..\\media\\images\\menu\\play3.png");

			// hiscore button.
			MenuButton* hiscore = new MenuButton(MM_HISCORE_BUT, xmid-155, ymid+50, "..\\media\\images\\menu\\hiscore1.png",
												"..\\media\\images\\menu\\hiscore2.png", "..\\media\\images\\menu\\hiscore3.png");

			// quit button.
			MenuButton* quit = new MenuButton(MM_QUIT_BUT, xmid-64, ymid+130, "..\\media\\images\\menu\\quit1.png",
												"..\\media\\images\\menu\\quit2.png", "..\\media\\images\\menu\\quit3.png");

			info.mMenuMgr->addMenuItem( title );
			info.mMenuMgr->addMenuItem( play );
			info.mMenuMgr->addMenuItem( hiscore );
			info.mMenuMgr->addMenuItem( quit );

			// load hi-scores.
			loadHighScores();

			mMainScoreboard->setRowText(0, "COMBO");
			mMainScoreboard->setRowText(1, " " );
			mMainScoreboard->setRowText(3, "MAX COMBO");
			mMainScoreboard->setRowText(4, " " );

			mMainScoreboard->setRowText(6, "HI-SCORE");
			if (info.mHiScores.size() > 0)
			{
				mMainScoreboard->setRowText(7, dbStr(info.mHiScores[0].score));
				mMainScoreboard->setRowText(8, info.mHiScores[0].name);
			}
			else
			{
				mMainScoreboard->setRowText(7, " ");
				mMainScoreboard->setRowText(8, " ");
			}

			
			fadeIn( 0.5f );

			info.mState = GS_MAIN_MENU;
		}


		// ------------------------------------------------------
		//	MAIN MENU
		// ------------------------------------------------------
		if (info.mState == GS_MAIN_MENU)
		{
			int click = dbMouseClick();

#ifdef _DEBUG
			char text[256];
			sprintf(text, "mousex:%d mousey:%d mouseclick:%d",info.mMouseX,info.mMouseY,click);
			dbText(5,5,text);
#endif

			//update the menu sytem.
			MenuManager::MenuInfoVector ret = info.mMenuMgr->updateMenu(info.mMouseX, info.mMouseY, click);
			bool quit = false;

			for (MenuManager::MenuInfoVectorIterator it = ret.begin(); it != ret.end(); it++)
			{
				if (it->ID == MM_PLAY_BUT)
				{
					//user has pressed the play game button
					delete info.mMenuMgr;
					info.mMenuMgr = NULL;
					info.mState = GS_START_GAME;
				}

				if (it->ID == MM_HISCORE_BUT)
				{
					//user has pressed the hi-score button.
					fadeOut( 0.5f );
					delete info.mMenuMgr;
					info.mMenuMgr = NULL;
					info.mState = GS_HISCORES;
				}

				if (it->ID == MM_QUIT_BUT)
				{
					fadeOut( 0.5f );
					quit = true;
				}
			}

			if ((dbEscapeKey()) || (quit))
			{
				delete info.mMenuMgr;
				info.mMenuMgr = NULL;
				break;
			}
		}


		// ------------------------------------------------------
		//	INITIALIZE THE GAME
		// ------------------------------------------------------
		if (info.mState == GS_START_GAME)
		{
			// ----------------
			// a few temp preview balls
			for (int i=0;i<8;i++)
			{
				BasicBall* ball = new BasicBall(info.mWorld, getColorInfo((Colors)dbRnd(C_MAGENTA)), 
					dVector(8.0f,1.0f+i,0.0f), info.mMatBall, info.mRootBallObjNum);
				if (i==0)
					ball->canPassPreview(true);

				ballCount++;
			}

			mMainScoreboard->setRowText(1, "0" );
			mMainScoreboard->setRowText(4, "0" );

			
			// setup the crosshair.
			mCrosshair = new UserCrosshair("..\\media\\models\\cursor1.x", "..\\media\\models\\cursor2.x", 
											dVector(0,0,-0.8f), dVector(-3.49f,-4.49f,-1.0f), dVector(3.49f,4.49f,1.0f));

			// setup overflow checker.
			mOverflowChecker = new OverflowChecker( info.mWorld, dVector(-4.0f,5.0f,0.0f), dVector(4.0f,5.0f,0.0f), 1.5 );

			// time to spawn next ball at the top
			info.mTimeToNextBall = 0.0f;
			info.mCurrentSpout = 0;

			info.mScore = 0;
			info.mComboCount = 0;
			info.mMaxCombo = 0;

			info.mLevel = 0;

			SoundManager::getSingleton().removeMusic( std::string("music") );
			SoundManager::getSingleton().addMusic( std::string("music"), std::string("..\\media\\sounds\\MUSIC1.mp3") );
			SoundManager::getSingleton().loopMusic( std::string("music") );

			info.mState = GS_INIT_LEVEL;
		}

        
		// ------------------------------------------------------
		//	BASIC PLAY LOOP
		// ------------------------------------------------------
		if ((info.mState == GS_PLAYING) || (info.mState == GS_INIT_LEVEL))
		{
			// press ecape to quit at anytime.
			if (dbEscapeKey()) 
			{
				while(dbEscapeKey()) {}
				info.mState = GS_STOP_PLAYING; 
			}

			// time elapsed in seconds...
			elapsedInSec += (float)((dbTimer() - oldtime) / 1000.0f);
			oldtime = dbTimer();

			char text[256];
#ifdef _DEBUG
			sprintf( text, "ballCount: %d", ballCount);
			dbText(5,5,text);
			sprintf( text, "level: %d", info.mLevel );
			dbText(5,15,text);
#endif

			if (info.mState == GS_PLAYING)
			{
				if (info.mScore > 0)
					sprintf( text, "%d", info.mScore );
				else
					sprintf( text, "GO!" );
			}
			else
			{
				if (info.mState == GS_INIT_LEVEL)
				{
					if (ballCount<30)
						sprintf( text, "READY..." );
					else
						sprintf( text, "SET..." );
				}
				else
					sprintf( text, "" );
			}
			mScoreBoard->setRowText( 0, text );

			// move crosshair object.
			dVector crossPos = mCrosshair->getPosition();
			crossPos = crossPos + dVector((float)(info.mMouseMoveX)*0.05f, (float)(-info.mMouseMoveY)*0.05f, 0.0);
			mCrosshair->setPosition( crossPos );
			crossPos = mCrosshair->getPosition();
			dbPositionLight( 2, crossPos.m_x, crossPos.m_y, crossPos.m_z );

	
			// -----------------------------------
			// clear any balls to check for deletion.
			info.mKillAllLikeMe = NULL;

			mColData.mBallsToCheck.clear();
			NewtonWorldForEachBodyDo( info.mWorld, BasicBall::_BasicBallClearCallback );
			info.mToDelete.clear();

			// loop internally multiple times to maintain proper physics FPS.
			int gameOver = 0;

			if ((elapsedInSec > physicsUpdate) && (elapsedInSec < 0.1f))
			{
				while (elapsedInSec > physicsUpdate)
				{
					OneGameLoop( physicsUpdate, mColData );
					elapsedInSec -= physicsUpdate;
					info.mTimeToNextBall -= physicsUpdate;
					mSkySphere->update( physicsUpdate, (info.mLevel+1) * 10.0f );
					mCrosshair->update( physicsUpdate );
					gameOver = mOverflowChecker->update( physicsUpdate );
				}
			}
			else
			{
				if (elapsedInSec < physicsUpdate)
				{
					// not enough time has passed to update.
				}
				else
				{
					// too much time has elapsed to handle, preform a single loop.
					OneGameLoop( physicsUpdate, mColData );
					elapsedInSec = 0.0f;
					info.mTimeToNextBall -= physicsUpdate;
					mSkySphere->update( physicsUpdate, (info.mLevel+1) * 10.0f );
					mCrosshair->update( physicsUpdate );
					gameOver = mOverflowChecker->update( physicsUpdate );
				}
			}

			// game over?
			if ((gameOver == 1) && (info.mState == GS_PLAYING))
			{
				info.mState = GS_GAME_LOST; 
			}
		

			// do we have any special deleting to do?
			if (info.mKillAllLikeMe)
			{
				// run a quick check for all balls of this color.
				NewtonWorldForEachBodyDo( info.mWorld, newtonWildCallback );
			}

			for (std::vector<BasicBall*>::iterator it = mColData.mBallsToCheck.begin(); it != mColData.mBallsToCheck.end(); it++)
			{
				BasicBall::BallContactMap touchCount;
				BasicBall::BallContactMap checked;
	
				RecursiveCheck( (*it), touchCount, checked );

				// checked the entire set of balls touching this ball.  how many were touching?
				if (touchCount.size() >= 4)
				{
					// delete this set, because at least 4 balls are touching!
					for (BasicBall::BallContactMapIterator kill = touchCount.begin(); kill != touchCount.end(); kill++)
					{
						BasicBall::BallContactMapIterator test = info.mToDelete.find( kill->first );
						if (test == info.mToDelete.end())
							info.mToDelete[ kill->first ] = kill->second;
					}
				}
			}


			int	ballsDeleted = 0;
			// we now have a full map of all bodies to delete.
			for (BasicBall::BallContactMapIterator kill = info.mToDelete.begin(); kill != info.mToDelete.end(); kill++)
			{
				// get the ball.
				BasicBall* ball = kill->second;

				if (ball->isPreviewBall())
					continue;

				dVector pos = ball->getPosition();
			
				// create a special effect when we destroy a ball.
				SFX_KillBall* effect = new SFX_KillBall( pos, info.mRootBallObjNum );
				SFXManager::getSingleton().addSFXObject( effect );

				delete ball;
				ballCount--;
				ballsDeleted++;
			}

			// before we delete, calculate the score!
			if (ballsDeleted > 0) 
			{ 
				// play clear sound
				SoundManager::getSingleton().playSound( std::string("clear") );

				info.mComboCount++;

				char text[256];
				sprintf( text, "%d", info.mComboCount );
				mMainScoreboard->setRowText( 1, text );

				if (info.mComboCount > info.mMaxCombo) 
				{ 
					sprintf( text, "%d", info.mMaxCombo );
					mMainScoreboard->setRowText( 4, text );
					info.mMaxCombo = info.mComboCount; 
				}
			
				// add to our score!!
				info.mScore += (ballsDeleted * (100 * (info.mComboCount^2)));

				// level up?
				info.mLevel = (info.mScore / 20000);
			}

		
			// user shot.
			if ((dbMouseClick()==1) && (mColData.mShootBall) && (mColData.mNextBall) && (info.mState != GS_INIT_LEVEL))
			{	
				mColData.mNextBall->canPassPreview(true);
				mColData.mNextBall->unfreeze();
				mColData.mNextBall = NULL;

				mColData.mShootBall->canPassShoot(true);
				mColData.mShootBall->unfreeze();
				mColData.mShootBall = NULL;
			
				mColData.mShootPosition = mCrosshair->getPosition();

				// create a new preview ball.
				spawnNewPreviewBall();

				// reset the combo counter.
				info.mComboCount = 0;

				// sound effect
				SoundManager::getSingleton().playSound( std::string("shoot") );

			}

			// spawn new balls
			if ((info.mTimeToNextBall < 0.0f))
			{
				spawnNewBall();
			}
		

		}


		// ------------------------------------------------------
		//	SHOW THE GAME OVER SCREEN
		// ------------------------------------------------------
		if (info.mState == GS_GAME_LOST)
		{
			// init a temp meu manager.
			info.mMenuMgr = new MenuManager();
			// game over screen.
			MenuPiece* gover = new MenuPiece( MM_TITLE, 160, 100, "..\\media\\images\\menu\\game_over.png" );
			info.mMenuMgr->addMenuItem( gover );

			// repeat until a key is pressed, or the user clicks the mouse.
			while ((dbMouseClick() == 0) && (dbReturnKey() == 0))
			{
				dbSync();
			}

			while (dbMouseClick() != 0) {}

			fadeOut( 0.5f );

			// delete.
			delete info.mMenuMgr;
			info.mMenuMgr = NULL;

			info.mState = GS_STOP_PLAYING;
		}

		// ------------------------------------------------------
		//	SHOW THE HI SCORES
		// ------------------------------------------------------
		if (info.mState == GS_HISCORES)
		{
			// make a quick list of the high scores as strings.
			std::vector<char*> scorelist;

			for (int i=0; i<info.mHiScores.size(); i++)
			{
				char* temp = NULL;
				temp = info.mHiScores[i].name;
				scorelist.push_back( temp );

				temp = dbStr( info.mHiScores[i].score );
				scorelist.push_back( temp );

				temp = "               ";
				scorelist.push_back( temp );
			}
	
			// create a huge scoreboard for high-scores.
			ScoreBoard* hiscores = new ScoreBoard(12,8,0.5f,1.0f,0.2f,"..\\media\\images\\font.png", dVector(-3.8f,6.2f,-1.5f), dVector(0,0,0) );

			// show hi-scores.
			hiscores->setRowText(0, "HIGH  SCORES");

			fadeIn( 0.5f );

			int row = -8;
			float scrolldelay = 0.4f;
			float timeleft = 0.0f;

			// repeat until a key is pressed, or the user clicks the mouse.
			while ((dbMouseClick() == 0) && (dbScanCode() == 0))
			{
				// time elapsed in seconds...
				elapsedInSec = (float)((dbTimer() - oldtime) / 1000.0f);
				oldtime = dbTimer();

				timeleft -= elapsedInSec;
				if (timeleft < 0.0f)
				{
					row = scrollHiScores( row, hiscores, scorelist );
					timeleft = scrolldelay;
				}

				dbSync();
			}

			while (dbMouseClick() != 0) {}

			fadeOut( 0.5f );

			// cleanup
			delete hiscores;

			info.mState = GS_LOAD_MENU;
		}


		// ------------------------------------------------------
		//	STOP PLAYING
		// ------------------------------------------------------
		if (info.mState == GS_STOP_PLAYING)
		{
			fadeOut( 0.5f );

			SoundManager::getSingleton().removeMusic( std::string("music") );
			SoundManager::getSingleton().addMusic( std::string("music"), std::string("..\\media\\sounds\\MENU_LOOP.mp3") );
			SoundManager::getSingleton().loopMusic( std::string("music") );

			info.mToDelete.clear();

			// kill all balls.
			NewtonWorldForEachBodyDo( info.mWorld, newtonDestroyBalls );

			for (BasicBall::BallContactMapIterator kill = info.mToDelete.begin(); kill != info.mToDelete.end(); kill++)
			{
				// get the ball.
				BasicBall* ball = kill->second;
				delete ball;
				ballCount--;
			}

			info.mToDelete.clear();

			delete mCrosshair;
			mCrosshair = NULL;

			delete mOverflowChecker;
			mOverflowChecker = NULL;

			info.mBombBall = NULL;
			info.mKillAllLikeMe =NULL;
			mColData.mNextBall = NULL;
			mColData.mShootBall = NULL;

			// do we have a new hi-score?
			updateHighScores();
			saveHighScores();
			
			info.mState = GS_LOAD_MENU;
		}
		
		// update the visuals.
		dbSync();
	}
	// -------------------------------------------------
	// -------------------------------------------------

#ifdef _DEBUG_LOG
	fprintf( debugLog, "Exiting....\n");
#endif


	// -----------------------------------------------------
	// CLEANUP
	delete mBallFrame;
	delete mSkySphere;
	delete mMainScoreboard;
	delete mScoreBoard;

	deleteColors();
	deleteGlobalResources();

#ifdef _DEBUG_LOG
	fprintf( debugLog, "BallFrame, skysphere, crosshair, scoreboards destroyed.\n");
#endif


	checkForLeaks();


	NewtonDestroy( info.mWorld );

#ifdef _DEBUG_LOG
	fprintf( debugLog, "NewtonWorld destroyed.  Really quitting now.\n");
	fclose( debugLog );
#endif
}


// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
void OneGameLoop( const float deltat, CollisionData& coldata )
{
	
	// -------------------------------------------
	// -------------------------------------------
	NewtonUpdate( WorldInfo::getSingleton().mWorld, deltat );
	// -------------------------------------------
	// -------------------------------------------
	SFXManager::getSingleton().updateAll( deltat );
	// -------------------------------------------
	// -------------------------------------------

}


void setupSpouts()
{
	WorldInfo& info = WorldInfo::getSingleton();

	Spout temp;
	
	// spout 0
	temp.mPos = dVector(-3.5f, 6.5f, 0.0f);
	temp.mVel = dVector(8.0f, -4.0f, 0.0f);
	info.mSpouts.push_back( temp );

	// spout 1
	temp.mPos = dVector(1.25f, 7.0f, 0.0f);
	temp.mVel = dVector(-3.0f, -4.0f, 0.0f);
	info.mSpouts.push_back( temp );

	// spout 2
	temp.mPos = dVector(-1.25f, 7.0f, 0.0f);
	temp.mVel = dVector(3.0f, -4.0f, 0.0f);
	info.mSpouts.push_back( temp );

	// spout 3
	temp.mPos = dVector(3.5f, 6.5f, 0.0f);
	temp.mVel = dVector(-8.0f, -4.0f, 0.0f);
	info.mSpouts.push_back( temp );
}


void spawnNewBall()
{
	WorldInfo& info = WorldInfo::getSingleton();;

	// get position and velocity.
	info.mCurrentSpout++;

	if (info.mCurrentSpout == info.mSpouts.size())
		info.mCurrentSpout = 0;

	Colors high = C_MAGENTA;
	if (info.mLevel > 0) { high = C_CYAN; }
	if (info.mLevel > 1) { high = C_WHITE; }
	if (info.mLevel > 2) { high = C_BLACK; }
	if (info.mLevel > 3) { high = C_BROWN; }
	
	BasicBall* ball = new BasicBall( info.mWorld, getColorInfo((Colors)dbRnd(high)), 
		info.mSpouts[info.mCurrentSpout].mPos, info.mMatBall, info.mRootBallObjNum );

	ball->setVelocity( info.mSpouts[ info.mCurrentSpout ].mVel );

	if (info.mState == GS_INIT_LEVEL)
	{
		if (ballCount < 40)
			info.mTimeToNextBall = 0.1f;
		else
		{
			info.mState = GS_PLAYING;
		}
	}
	else
	{
		info.mTimeToNextBall = (1.0f - (info.mLevel * 0.1f));
		if (info.mTimeToNextBall <= 0.2f) { info.mTimeToNextBall = 0.2f; }
	}


	ballCount++;


}


void spawnNewPreviewBall()
{
	WorldInfo& info = WorldInfo::getSingleton();

	Colors high = C_MAGENTA;
	if (info.mLevel > 0) { high = C_CYAN; }
	if (info.mLevel > 1) { high = C_WHITE; }
	if (info.mLevel > 2) { high = C_BLACK; }
	if (info.mLevel > 3) { high = C_BROWN; }

	Colors thecolor = (Colors)dbRnd( high );

	// small chance you'll get a bomb or wild.
	if (dbRnd(50) <= info.mLevel) { thecolor = C_WILD; }
	if (dbRnd(30) <= info.mLevel) { thecolor = C_BOMB; }

	BasicBall* ball = new BasicBall(info.mWorld, getColorInfo( thecolor ), 
					dVector(8.0,2.0,0.0), info.mMatBall, info.mRootBallObjNum);
	
	ball->isPreviewBall( true );
	ballCount++;

}


void setupColors()
{
	WorldInfo& info = WorldInfo::getSingleton();;

	// create the basic color information.
	ColorInfo temp;

	// RED.
	temp.mEnumVal = C_RED;
	temp.mColor = dbRgb( 255,0,0 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\red.png", temp.mImgNum );
	info.mColors[ C_RED ] = temp;

	// GREEN.
	temp.mEnumVal = C_GREEN;
	temp.mColor = dbRgb( 0,255,0 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\green.png", temp.mImgNum );
	info.mColors[ C_GREEN ] = temp;

	// BLUE.
	temp.mEnumVal = C_BLUE;
	temp.mColor = dbRgb( 0,0,255 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\blue.png", temp.mImgNum );
	info.mColors[ C_BLUE ] = temp;

	// YELLOW.
	temp.mEnumVal = C_YELLOW;
	temp.mColor = dbRgb( 255,255,0 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\yellow.png", temp.mImgNum );
	info.mColors[ C_YELLOW ] = temp;

	// MAGENTA.
	temp.mEnumVal = C_MAGENTA;
	temp.mColor = dbRgb( 255,0,255 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\magenta.png", temp.mImgNum );
	info.mColors[ C_MAGENTA ] = temp;

	// CYAN.
	temp.mEnumVal = C_CYAN;
	temp.mColor = dbRgb( 0,255,255 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\cyan.png", temp.mImgNum );
	info.mColors[ C_CYAN ] = temp;

	// WHITE.
	temp.mEnumVal = C_WHITE;
	temp.mColor = dbRgb( 255,255,255 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\white.png", temp.mImgNum );
	info.mColors[ C_WHITE ] = temp;

	// BLACK.
	temp.mEnumVal = C_BLACK;
	temp.mColor = dbRgb( 30,305,30 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\black.png", temp.mImgNum );
	info.mColors[ C_BLACK ] = temp;

	// BROWN.
	temp.mEnumVal = C_BROWN;
	temp.mColor = dbRgb( 128,100,80 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\brown.png", temp.mImgNum );
	info.mColors[ C_BROWN ] = temp;

	// WILD.
	temp.mEnumVal = C_WILD;
	temp.mColor = dbRgb( 128,128,128 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\wild.png", temp.mImgNum );
	info.mColors[ C_WILD ] = temp;

	// BOMB.
	temp.mEnumVal = C_BOMB;
	temp.mColor = dbRgb( 128,0,0 );
	temp.mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( "..\\media\\images\\bomb.png", temp.mImgNum );
	info.mColors[ C_BOMB ] = temp;

}

void deleteColors()
{
	WorldInfo& info = WorldInfo::getSingleton();

	for (WorldInfo::ColorInfoMapIterator it = info.mColors.begin(); it != info.mColors.end(); it++)
	{
		dbDeleteImage( it->second.mImgNum );
	}
}


ColorInfo getColorInfo( Colors color )
{
	WorldInfo& info = WorldInfo::getSingleton();;

	WorldInfo::ColorInfoMapIterator it = info.mColors.find( color );

	if (it != info.mColors.end())
		return it->second;

	ColorInfo ret;
	ret.mColor = 0;
	ret.mImgNum = 0;

	return ret;

}


// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
void RecursiveCheck( BasicBall* checkme, BasicBall::BallContactMap& toDelete, BasicBall::BallContactMap& checked )
{
	// first, add this ball to the list of balls checked.
	BasicBall::BallContactMapIterator test = checked.find( checkme->mObjNum );

	if (test != checked.end())
	{
		// already checked this one!!
		return;
	}
	else
	{
		// haven't checked this one yet, so add it to the list.
		checked[ checkme->mObjNum ] = checkme;

		// add this to the delete list.
		test = toDelete.find( checkme->mObjNum );

		if (test == toDelete.end())
		{
			toDelete[ checkme->mObjNum ] = checkme;
		}

		//now check all balls in my list.
		BasicBall::BallContactMap testmap = checkme->getContactMap();
		for (test = testmap.begin(); test != testmap.end(); test++)
			RecursiveCheck( test->second, toDelete, checked );
	}
}


// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
void _cdecl newtonLeaveWorld( const NewtonBody* body )
{
	// get the physics object.
	PhysicsObject* obj = (PhysicsObject*)NewtonBodyGetUserData( body );

	if (obj->getType() == BT_BALL)
	{
		//this is a ball, destroy it!
		BasicBall* ball = (BasicBall*)obj;

		// subtract score slightly for each lost ball.
		WorldInfo::getSingleton().mScore -= 75;

		delete ball;
		ballCount--;
	}
}




void _cdecl newtonWildCallback( const NewtonBody* body )
{
	WorldInfo& info = WorldInfo::getSingleton();

	PhysicsObject* obj = (PhysicsObject*)NewtonBodyGetUserData( body );

	if (obj->getType() == BT_BALL)
	{
		BasicBall* ball = (BasicBall*)obj;

		if ((ball->getColorInfo().mEnumVal == info.mKillAllLikeMe->getColorInfo().mEnumVal) || (ball->getColorInfo().mEnumVal == C_WILD))
		{
			
			// make a special effect here.
			SFX_KillBall* effect = new SFX_KillBall( ball->getPosition(), info.mRootBallObjNum );
			SFXManager::getSingleton().addSFXObject( effect );

			//delete the ball.
			BasicBall::BallContactMapIterator it = info.mToDelete.find( ball->mObjNum );
			if (it == info.mToDelete.end())
			{
				info.mToDelete[ ball->mObjNum ] = ball;
			}
			
		}
	}
}

void _cdecl newtonBombCallback( const NewtonBody* body )
{
	WorldInfo& info = WorldInfo::getSingleton();

	PhysicsObject* obj = (PhysicsObject*)NewtonBodyGetUserData( body );

	if (obj->getType() == BT_BALL)
	{
		BasicBall* ball = (BasicBall*)obj;

		
	}
}



void _cdecl newtonDestroyBalls( const NewtonBody* body )
{
	PhysicsObject* obj = (PhysicsObject*)NewtonBodyGetUserData( body );

	if (obj->getType() == BT_BALL)
	{
		BasicBall* ball = (BasicBall*)obj;
		WorldInfo::getSingleton().mToDelete[ ball->mObjNum ] = ball;
	}
}

// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
// NEWTON MATERIAL CALLBACKS
int _cdecl newtonContactBegin(const NewtonMaterial* material, const NewtonBody* body0, const NewtonBody* body1 )
{
	// begin... in this function we just save the 2 bodies involved...
	CollisionData* coldata = (CollisionData*)NewtonMaterialGetMaterialPairUserData( material );

	coldata->mBody0 = (NewtonBody*)body0;
	coldata->mBody1 = (NewtonBody*)body1;

	coldata->mHitSound = coldata->mBombSound = coldata->mWildSound = false;

	return 1;
}

int _cdecl newtonContactProcess( const NewtonMaterial* material, const NewtonContact* contact )
{
	// get body info.
	CollisionData* coldata = (CollisionData*)NewtonMaterialGetMaterialPairUserData( material );

	// first get our phyicsobjects.
	PhysicsObject* obj0 = (PhysicsObject*)NewtonBodyGetUserData( coldata->mBody0 );
	PhysicsObject* obj1 = (PhysicsObject*)NewtonBodyGetUserData( coldata->mBody1 );

	if ((obj0->getType() == BT_BALL) && (obj1->getType() == BT_BALL))
	{
		NewtonMaterialSetContactElasticity( material, 0.4f );
		NewtonMaterialSetContactSoftness( material, 0.3f );

		float speed = NewtonMaterialGetContactNormalSpeed( material, contact );
		if (speed > 3.0f)
			coldata->mHitSound = true;
		

		// Ball vs. Ball collision!
		BasicBall* ball0 = (BasicBall*)obj0;
		BasicBall* ball1 = (BasicBall*)obj1;

		// remove flying ball ptr.
		BasicBall* flying = WorldInfo::getSingleton().mFlyingBall;
		if (ball0 == flying) { WorldInfo::getSingleton().mFlyingBall = NULL; }
		if (ball1 == flying) { WorldInfo::getSingleton().mFlyingBall = NULL; }

		// same color?
		if (ball0->getColorInfo().mColor == ball1->getColorInfo().mColor)
		{
			// add each other to the list...
			if (ball1->possibleTouch( ball0 ) >= 3)
			{
				// add these 2 balls to the check list.
				coldata->mBallsToCheck.push_back( ball0 );
				coldata->mBallsToCheck.push_back( ball1 );
			}
			else
			{
	            if (ball0->possibleTouch( ball1 ) >= 3)
				{
					// add these 2 balls to the check list.
					coldata->mBallsToCheck.push_back( ball0 );
					coldata->mBallsToCheck.push_back( ball1 );
				}
			}
		}

		BasicBall* bomb = NULL;
		if (ball0->getColorInfo().mEnumVal == C_BOMB) { bomb = ball0; }
		if (ball1->getColorInfo().mEnumVal == C_BOMB) { bomb = ball1; }

		if (bomb)
		{
			if (!bomb->isPreviewBall())
			{
				WorldInfo::getSingleton().mToDelete[ bomb->mObjNum ] = bomb;
				SFX_Bomb* bombeffect = new SFX_Bomb( bomb, bomb->getPosition(), 15 );
				SFXManager::getSingleton().addSFXObject( bombeffect );

				coldata->mBombSound = true;
			}
		}

		//quick check for wilds.
		BasicBall* wild = NULL;
		BasicBall* notwild = NULL;
		if (ball0->getColorInfo().mEnumVal == C_WILD) { wild = ball0; notwild = ball1; }
		if (ball1->getColorInfo().mEnumVal == C_WILD) { wild = ball1; notwild = ball0; }

		if (wild)
		{
			if (!wild->isPreviewBall())
			{
				WorldInfo::getSingleton().mKillAllLikeMe = notwild;
				coldata->mWildSound = true;
			}
		}

		BasicBall* thrown = NULL;
		BasicBall* basic = NULL;

		if (ball0->canPassFront())
		{
			thrown = ball0;
			basic = ball1;
		}
		else
		{
			if (ball1->canPassFront())
			{
				thrown = ball1;
				basic = ball0;
			}
		}

		if (thrown)
		{
			// knock other ball out!
			basic->canPassBack(true);

			// remove the special abilities of this ball?
			thrown->mBallsHit++;
			if (thrown->mBallsHit > 1)
			{
				thrown->canPassFront(false);
			}

		}

		return 1;
	}


	// --------------------------------------------------------------------------------------------------------
	// --------------------------------------------------------------------------------------------------------
	// not a ball-on-ball collision, so one of these must be the frame...
	NewtonMaterialSetContactElasticity( material, 0.4f );
	NewtonMaterialSetContactSoftness( material, 0.5f );
	
	BallFrame* frame = NULL;
	BasicBall* theball = NULL;


	if (obj0->getType() == BT_FRAME)
	{
		frame = (BallFrame*)obj0;
		if (obj1->getType() == BT_BALL)
		{
			theball = (BasicBall*)obj1;
		}
		else
		{
			// error!
			return 0;
		}
	}
	else
	{
		if (obj1->getType() == BT_FRAME)
		{
			frame = (BallFrame*)obj1;
			if (obj0->getType() == BT_BALL)
			{
				theball = (BasicBall*)obj0;
			}
			else
			{
				// error!
				return 0;
			}
		}
		else
		{
			// error!
			return 0;
		}
	}

	// clear the flying ball.
	if (theball == WorldInfo::getSingleton().mFlyingBall) { WorldInfo::getSingleton().mFlyingBall = NULL; }

	// if we've gotten to this point, we have found the ball and the frame involved in the collision, and can go on from here.
	// now get what part of the frame we've hit.
	int colID = NewtonMaterialGetBodyCollisionID(material, frame->mBody);

	// have we hit the front wall?
	if (colID == (int)CP_FRONT)
	{
		if (theball->canPassFront())
		{
			// this ball can pass the front!
			return 0;
		}
		else
		{
			// cannot pass the front!
			return 1;
		}
	}

	// have we hit the back?
	if (colID == (int)CP_BACK)
	{
		if (theball->canPassBack())
		{
			// this ball can pass the back!
			return 0;
		}
		else
		{
			// cannot pass the back!
			theball->canPassFront(false);
			return 1;
		}
	}

	// have we hit the preview stopper?
	if (colID == (int)CP_PREVIEW)
	{
		if (theball->canPassPreview())
		{
			// this ball can pass the back!
			return 0;
		}
		else
		{
			// cannot pass the back!
			coldata->mNextBall = theball;

			return 1;
		}
	}

	// have we hit the shoot spot?
	if (colID == (int)CP_SHOOT)
	{
		if (theball->canPassShoot())
		{
			coldata->mShootBall = NULL;
			return 0;
		}
		else
		{
			coldata->mShootBall = theball;
			return 1;
		}
	}

	if (colID == (int)CP_SHOOT_TRIGGER)
	{
		WorldInfo& info = WorldInfo::getSingleton();

		dVector pos(0,-1.0f,-12.0f);
		// get velocity...
		dVector vel = (coldata->mShootPosition - pos).Scale(5.5) + dVector(0.0,1.3,0.0);

		theball->setPosition(pos);
		theball->setVelocity(vel);
		theball->unfreeze();
		theball->canPassFront(true);
		theball->canPassPreview(false);
		theball->canPassShoot(false);
		theball->canPassBack(false);
		theball->isPreviewBall(false);

		info.mFlyingBall = theball;

		// add a new smoke system to the sfx manager.
		int smokenum = info.mRootSmoke1ImgNum;
		if (theball->getColorInfo().mEnumVal == C_BOMB) { smokenum = info.mRootSmoke2ImgNum; }
		SFX_Smoke* smoke = new SFX_Smoke( 35.0f, smokenum );
		SFXManager::getSingleton().addSFXObject( smoke );
	}

	
	return 1;
}


void _cdecl newtonContactEnd( const NewtonMaterial* material )
{
	CollisionData* coldata = (CollisionData*)NewtonMaterialGetMaterialPairUserData( material );

	// hit sound	
	if (coldata->mHitSound)
		SoundManager::getSingleton().playSound( std::string("hit") );

	// bomb sound	
	if (coldata->mBombSound)
		SoundManager::getSingleton().playSound( std::string("bomb") );

	// wild sound	
	if (coldata->mWildSound)
		SoundManager::getSingleton().playSound( std::string("wild") );

}




void deleteGlobalResources()
{
	WorldInfo& info = WorldInfo::getSingleton();
	
	dbDeleteObject( info.mRootBallObjNum );

	// root smoke particle images.
	dbDeleteImage( info.mRootSmoke1ImgNum );
	dbDeleteImage( info.mRootSmoke2ImgNum );
}



void loadHighScores()
{
	// open the file.
	dbOpenToRead( 1, "..\\hiscores.txt" );

	WorldInfo::getSingleton().mHiScores.clear();

	// read in all high scores.
	char* text = NULL;

	text = dbReadString( 1 );

	while( dbLen(text) > 0 )
	{
		HiScore temp;
		temp.name = text;
		text = dbReadString( 1 );
		temp.score = dbValF( text );
		WorldInfo::getSingleton().mHiScores.push_back( temp );
		text = dbReadString( 1 );
	}

	dbCloseFile( 1 );
}


void saveHighScores()
{
	WorldInfo& info = WorldInfo::getSingleton();

	if (dbFileExist( "..\\hiscores.txt" ))
		dbDeleteFile( "..\\hiscores.txt" );

	dbOpenToWrite( 1, "..\\hiscores.txt" );


	for (int i=0; i<info.mHiScores.size(); i++)
	{
		char* temp = NULL;
		temp = info.mHiScores[i].name;
		dbWriteString( 1, temp );

		temp = dbStr( info.mHiScores[i].score );
		dbWriteString( 1, temp );
	}

	dbCloseFile( 1 );
}

void updateHighScores()
{
	WorldInfo& info = WorldInfo::getSingleton();

	// add this score to the hi-score list.
	std::vector<HiScore> newHiScores;

	bool insertedScore = false;

	for (int i=0; i<info.mHiScores.size(); i++)
	{
		// is our score higher than this one?
		if (!insertedScore)
		{
			if (info.mScore > info.mHiScores[i].score)
			{
				//insert here!
				HiScore temp;
				temp.name = "";
				temp.score = info.mScore;

				// get the player's name!!
				ScoreBoard* tempboard = new ScoreBoard(10,3,0.4f,1.0f,0.2f,"..\\media\\images\\font.png", dVector(-2.8f,1.0f,-2.0f), dVector(0,0,0) );
				tempboard->setRowText(0,"TYPE NAME:");
				tempboard->setRowText(1,dbStr(temp.score));

				fadeIn( 0.2f );

				bool done = false;
				
				char key[256];
				char final[256];
				sprintf( final, "" );

				while (!done)
				{
					int scan = dbScanCode();
					
					if (scan != 0)
					{
						while (dbScanCode() != 0)
						{ 
							if (dbReturnKey()) { done = true; }
						}

						if (scan != 14)
						{
							getASCII( scan, key );
							sprintf( final, "%s%s", temp.name, key );
							sprintf( temp.name, "%s", final );

							
						}
						else
						{
							if (dbLen(temp.name) > 0)
							{
								temp.name = dbLeft( temp.name, dbLen(temp.name)-1 );
							}
						}
					}

					temp.name = dbLeft( temp.name, 10 );
					tempboard->setRowText(2, temp.name );

					dbSync();
				}

				newHiScores.push_back(temp);
				insertedScore = true;

				fadeOut( 0.2f );

				delete tempboard;
			}
		}

		// add existing score.
		HiScore temp = info.mHiScores[i];

		newHiScores.push_back( temp );
	}

	info.mHiScores.clear();
	info.mHiScores = newHiScores;
}

		

int scrollHiScores( int toprow, ScoreBoard* board, std::vector<char*>& textvector )
{
	
	WorldInfo& info = WorldInfo::getSingleton();

	int startRow = 2;
	int lastRow = 8;

	int scorecount = info.mHiScores.size();
	int offset = 0;

	for (int row = startRow; row <= lastRow; row++)
	{
		// get the text for this row.
		int thisrow = toprow + row;

		if ((thisrow >= 0) && (thisrow < textvector.size()))
		{
			// worth drawing.
			board->setRowText( row, textvector[thisrow] );
		}
		else
		{
			board->setRowText( row, "                " );
		}

	}

	if (toprow > (startRow + scorecount*3))
	{
		toprow = -lastRow;
	}

	return ++toprow;
}



void setupFader( int color )
{
	WorldInfo& info = WorldInfo::getSingleton();

	// make a small sprite of the specified color.
	int mem = Helpers::getOpenMemblockNum();
	int sizeX = 32;
	int sizeY = 32;

	int memsize = (sizeX * sizeY * 4) + 12;
	dbMakeMemblock( mem, memsize );

	// fill the data.
	int offset = 0;

	dbWriteMemblockDWORD( mem, offset, sizeX );		offset += 4;
	dbWriteMemblockDWORD( mem, offset, sizeY );		offset += 4;
	dbWriteMemblockDWORD( mem, offset, 32 );		offset += 4;

	for (int y=0; y<sizeY; y++)
	{
		for (int x=0; x<sizeX; x++)
		{
			dbWriteMemblockDWORD( mem, offset, color );		offset += 4;
		}
	}

	// make an image from this.
	if (info.mFadeSpriteImg != 0)
	{
		if (dbImageExist( info.mFadeSpriteImg ))
			dbDeleteImage( info.mFadeSpriteImg );
	}

	info.mFadeSpriteImg = Helpers::getOpenImageNum();
	dbMakeImageFromMemblock( info.mFadeSpriteImg, mem );

	// delete memblock.
	dbDeleteMemblock( mem );

	if (info.mFadeSprite == 0)
	{
		// make the sprite.
		info.mFadeSprite = Helpers::getOpenSpriteNum();

		dbSprite( info.mFadeSprite, 0, 0, info.mFadeSpriteImg );

		//scale!
		float xscale = (dbScreenWidth() / dbSpriteWidth(info.mFadeSprite)) * 105.0f;
		dbScaleSprite( info.mFadeSprite, xscale );
		dbSetSpritePriority( info.mFadeSprite, 500 );
		dbSetSpriteAlpha( info.mFadeSprite, 0 );
	}
}



void fadeOut( float fadeTimeInSec )
{
	//loop through, changing the alpha.
	int oldtime = dbTimer();

	float alphaChangePerSec = 255.0f / fadeTimeInSec;

	float alpha = 0.0f;

	// loop through!
	bool done = false;

	while (!done)
	{
		//get time elapsed.
		float elapsedInSec = (dbTimer() - oldtime) / 1000.0f;
		oldtime = dbTimer();

		alpha += alphaChangePerSec * elapsedInSec;
		if (alpha > 255.0f) { alpha = 255.0f; done = true; }

		dbSetSpriteAlpha( WorldInfo::getSingleton().mFadeSprite, (int)alpha );

		dbSync();
	}
}

		
void fadeIn( float fadeTimeInSec )
{
	//loop through, changing the alpha.
	int oldtime = dbTimer();

	float alphaChangePerSec = 255.0f / fadeTimeInSec;

	float alpha = 255.0f;

	// loop through!
	bool done = false;

	while (!done)
	{
		//get time elapsed.
		float elapsedInSec = (dbTimer() - oldtime) / 1000.0f;
		oldtime = dbTimer();

		alpha -= alphaChangePerSec * elapsedInSec;
		if (alpha < 0.0f) { alpha = 0.0f; done = true; }

		dbSetSpriteAlpha( WorldInfo::getSingleton().mFadeSprite, (int)alpha );

		dbSync();
	}
}
	
			


void checkForLeaks()
{
	int mObjFound = 0;
	int mImgFound = 0;

	for (int i=0; i<30000; i++)
	{
		if (dbObjectExist(i))
		{
			dbDeleteObject(i);
			mObjFound++;
		}

		if (dbImageExist(i))
		{
			dbDeleteImage(i);
			mImgFound++;
		}
	}

#ifdef _DEBUG_LOG
	fprintf( debugLog, "LEAK REPORT:\n");
	fprintf( debugLog, "undeleted objects: %d\n", mObjFound );
	fprintf( debugLog, "undeleted images:  %d\n", mImgFound );
	fprintf( debugLog, "\n");
#endif
}














void getASCII( int scancode, char* ascii )
{
	char* toprow = "QWERTYUIOP[]";
	char* midrow = "ASDFGHJKL;@";
	char* botrow = "ZXCVBNM,./";
	
	if ((scancode >= 16) && (scancode  <= 27))
	{
		sprintf( ascii, "%s", dbMid( toprow, (scancode - 15) ) );
		return;
	}

	if ((scancode >= 30) && (scancode  <= 40))
	{
		sprintf( ascii, "%s", dbMid( midrow, (scancode - 29) ) );
		return;
	}

	if ((scancode >= 44) && (scancode  <= 53))
	{
		sprintf( ascii, "%s", dbMid( botrow, (scancode - 43) ) );
		return;
	}

	if (scancode == 57)
	{
		sprintf( ascii, " " );
		return;
	}



	sprintf(ascii, "" );
		

}