#include "MenuSystem.h"

MenuPiece::MenuPiece(int ID, int x, int y, char* imgfile)
{
	mID = ID;

	mX = x;
	mY = y;

	mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( imgfile, mImgNum );

	mSpriteNum = Helpers::getOpenSpriteNum();
	dbSprite( mSpriteNum, mX, mY, mImgNum );

	mWidth = dbSpriteWidth( mSpriteNum );
	mHeight = dbSpriteHeight( mSpriteNum );
}

MenuPiece::~MenuPiece()
{
	dbDeleteSprite( mSpriteNum );
	dbDeleteImage( mImgNum );
}

void MenuPiece::setXY(int x, int y)
{
	mX = x;
	mY = y;

	dbSprite( mSpriteNum, mX, mY, mImgNum );
}



MenuButton::MenuButton(int ID, int x, int y, char* imgfile, char* rolloverimg, char* clickimg ) : MenuPiece(ID,x,y,imgfile)
{
	//load extra images.
	mRollOverImg = Helpers::getOpenImageNum();
	dbLoadImage( rolloverimg, mRollOverImg );

	mClickImg = Helpers::getOpenImageNum();
	dbLoadImage( clickimg, mClickImg );

	mMouseDown = false;
}

MenuButton::~MenuButton()
{
	dbDeleteImage( mRollOverImg );
	dbDeleteImage( mClickImg );
}


int MenuButton::_update(int x, int y, int buttonState )
{
	if (getCollision(x,y))
	{
		//mouse is over the button.
		if (buttonState == 1)
		{
			//set click image.
			dbSprite( getSpriteNum(), getX(), getY(), mClickImg );
			mMouseDown = true;
			return 0;
		}
		else
		{
			//mouse roll-over.
			dbSprite( getSpriteNum(), getX(), getY(), mRollOverImg );
			if (mMouseDown)
			{
				mMouseDown = false;
				return 1;
			}
			else
			{
				mMouseDown = false;
				return 0;
			}
		}
	}

	//nothing.
	mMouseDown = false;
	dbSprite( getSpriteNum(), getX(), getY(), getImgNum() );
	return 0;
}


// -----------------------------------------
MenuManager::MenuManager()
{
	mCursor = NULL;
	mScreenWidth = 0;
	mScreenHeight = 0;
}

MenuManager::~MenuManager()
{
	// destroy all menu pieces.
	while (mMenuPieces.size() > 0)
	{
		MenuPieceMapIterator it = mMenuPieces.begin();

		MenuPiece* piece = it->second;

		mMenuPieces.erase( it );

		delete piece;
	}

	mCursor = NULL;
}


void MenuManager::addMenuItem( MenuPiece* item )
{
	MenuPieceMapIterator it = mMenuPieces.find( item->getID() );

	if (it == mMenuPieces.end())
	{
		mMenuPieces[ item->getID() ] = item;
	}
}


MenuManager::MenuInfoVector MenuManager::updateMenu()
{
	int x, y, button;

	// cursor exists?
	if (mCursor)
	{
		x = mCursor->getX() + dbMouseMoveX();
		y = mCursor->getY() + dbMouseMoveY();
		button = dbMouseClick();
	}
	else
	{
		x = y = button = 0;
	}


	return updateMenu( x, y, button );
}


void MenuManager::addCursor(char* img)
{
	if (!mCursor)
	{
		mCursor = new MenuPiece( -1, 0, 0, img );
		mCursor->setPriority( 1000 );
		addMenuItem( mCursor );
	}
}

MenuManager::MenuInfoVector MenuManager::updateMenu( int x, int y, int buttonState )
{
	if (mCursor)
	{
		if (x > mScreenWidth) { x = mScreenWidth; }
		if (y > mScreenHeight) { y = mScreenHeight; }
		if (x < 0) { x = 0; }
		if (y < 0) { y = 0; }

		mCursor->setXY( x, y );
	}
	// update all menu items.
	MenuInfoVector ret;

	for (MenuPieceMapIterator it = mMenuPieces.begin(); it != mMenuPieces.end(); it++)
	{
		MenuInfo temp;

		temp.retVal = it->second->_update(x,y,buttonState);

		if (temp.retVal != 0)
		{
			temp.ID = it->first;
			temp.ptr = it->second;
			ret.push_back( temp );
		}
	}

	return ret;
}