/*
	ConnectMore project

	by Walaber

	MenuSystem- set of classes for a simple menu system.
*/

#ifndef _MENUSYSTEM_INCLUDED_
#define _MENUSYSTEM_INCLUDED_

#include "Prereqs.h"
#include "HelperFunctions.h"
#include <Newton.h>
#include <DarkSDK.h>

// MenuPiece - basic menu item, contains an image and a location.
class MenuPiece
{
public:
	// constructor
	MenuPiece( int ID, int x, int y, char* imgfile );

	//destructor
	virtual ~MenuPiece();

	// basic information.
	int getID() { return mID; }

	int getX() { return mX; }
	int getY() { return mY; }

	int getWidth() {  return mWidth; }
	int getHeight() { return mHeight; }

	int getImgNum() { return mImgNum; }
	int getSpriteNum() { return mSpriteNum; }

	bool getCollision( int x, int y )
	{
		return ( ((x>=mX)&&(x<=(mX+mWidth))) && ((y>=mY)&&(y<=(mY+mHeight))) );
	}

	// set new position
	void setXY(int x, int y);

	// set priority.
	void setPriority( int priority ) { dbSetSpritePriority( mSpriteNum, priority ); }

	// function that should be updated by the item.
	virtual int _update( int x, int y, int buttonState ) { return 0; }

private:
	int mID;

	int mImgNum;
	int mSpriteNum;

	int mX;
	int mY;

	int mHeight;
	int mWidth;
};


// basic menu button.
class MenuButton : public MenuPiece
{
public:
	MenuButton( int ID, int x, int y, char* imgfile, char* rolloverimg, char* clickimg );

	~MenuButton();

	// simple update function.
	int _update( int x, int y, int buttontate );

private:
	int mRollOverImg;
	int mClickImg;

	bool mMouseDown;
	
};


// menu manager. holds a bunch of menu items, and controls them.
class MenuManager
{
public:
	struct MenuInfo
	{
		int ID;
		int retVal;
		MenuPiece* ptr;
	};

	typedef std::vector<MenuInfo> MenuInfoVector;
	typedef std::vector<MenuInfo>::iterator MenuInfoVectorIterator;
	
	MenuManager();
	~MenuManager();

	// set screen size.
	void setScreenSize( int width, int height ) { mScreenWidth = width; mScreenHeight = height; }

	// add a menu item to the menu.
	void addMenuItem( MenuPiece* item );

	// add a cursor.
	void addCursor( char* img );

	// update the entire menu. returns a list of pieces that returned something other than 0.
	MenuInfoVector updateMenu();

	// update using your own cursor coordinates.
	MenuInfoVector updateMenu( int x, int y, int buttonState );


private:

	MenuPiece* mCursor;
	int mScreenWidth;
	int mScreenHeight;

	typedef std::map<int,MenuPiece*> MenuPieceMap;
	typedef std::map<int,MenuPiece*>::iterator MenuPieceMapIterator;

	MenuPieceMap mMenuPieces;
};

	


#endif	// _MENUSYSTEM_INCLUDED_
	