#include ".\PhysicsObject.h"


PhysicsObject::PhysicsObject()
{
	mObjNum = -1;
	mWorld = NULL;
	mBody = NULL;
	mType = -1;
}


PhysicsObject::~PhysicsObject()
{
	// delete the visual object, as well as the rigid body.
	if (mObjNum != -1)
		dbDeleteObject( mObjNum );

	if ((mBody) && (mWorld))
		NewtonDestroyBody( mWorld, mBody );

}






dVector PhysicsObject::getPosition()
{
	// use Newton for this.
	float matrix[16];

	NewtonBodyGetMatrix( mBody, matrix );

	return dVector(matrix[12], matrix[13], matrix[14]);
}

void PhysicsObject::setPosition( dVector& pos )
{
	// use Newton.
	float matrix[16];

	NewtonBodyGetMatrix( mBody, matrix );

	matrix[12] = pos.m_x;
	matrix[13] = pos.m_y;
	matrix[14] = pos.m_z;

	NewtonBodySetMatrix( mBody, matrix );

	dbPositionObject( mObjNum, pos.m_x, pos.m_y, pos.m_z );
}

dMatrix PhysicsObject::getMatrix()
{
	float matrix[16];

	NewtonBodyGetMatrix( mBody, matrix );

	return dMatrix( dVector(matrix[0], matrix[1], matrix[2], matrix[3]),
					dVector(matrix[4], matrix[5], matrix[6], matrix[7]),
					dVector(matrix[8], matrix[9], matrix[10], matrix[11]),
					dVector(matrix[12], matrix[13], matrix[14], matrix[15]) );
}


void PhysicsObject::setMatrix( dMatrix& mat )
{
	NewtonBodySetMatrix( mBody, &mat.m_front.m_x );
	dbPositionObject( mObjNum, mat.m_posit.m_x, mat.m_posit.m_y, mat.m_posit.m_z );
}


dVector PhysicsObject::getEulerAngles()
{
	float matrix[16];
	dVector ret;
	
	NewtonBodyGetMatrix( mBody, matrix );
	NewtonGetEulerAngle( matrix, &ret.m_x );

	// convert to degrees.
	ret = ret.Scale(180.0f / 3.14159f);

	return ret;
}

void PhysicsObject::setEulerAngles( dVector& angles )
{
	dVector pos = getPosition();

	float matrix[16];
	
	// convert to radians.
	angles = angles.Scale(3.14159f / 180.0f);

	NewtonSetEulerAngle( &angles.m_x, matrix );

	matrix[12] = pos.m_x;
	matrix[13] = pos.m_y;
	matrix[14] = pos.m_z;

	NewtonBodySetMatrix( mBody, matrix );
	dbPositionObject( mObjNum, pos.m_x, pos.m_y, pos.m_z );
}


dVector PhysicsObject::getVelocity()
{
	dVector vel;

	NewtonBodyGetVelocity( mBody, &vel.m_x );

	return vel;
}

void PhysicsObject::setVelocity( dVector& vel )
{
	NewtonBodySetVelocity( mBody, &vel.m_x );
}


float PhysicsObject::getMass()
{
	float mass;
	dVector inertia;

	NewtonBodyGetMassMatrix( mBody, &mass, &inertia.m_x, &inertia.m_y, &inertia.m_z );

	return mass;
}

void PhysicsObject::getMassMatrix( float& mass, dVector& inertia )
{
	NewtonBodyGetMassMatrix( mBody, &mass, &inertia.m_x, &inertia.m_y, &inertia.m_z );
}

void PhysicsObject::setMassMatrix( float& mass, dVector& inertia )
{
	NewtonBodySetMassMatrix( mBody, mass, inertia.m_x, inertia.m_y, inertia.m_z );
}

void PhysicsObject::freeze()
{
	NewtonWorldFreezeBody( mWorld, mBody );
}

void PhysicsObject::unfreeze()
{
	NewtonWorldUnfreezeBody( mWorld, mBody );
}






void PhysicsObject::_doPhysicsWiring()
{
	// store the user data!
	NewtonBodySetUserData( mBody, this );

	// set transform callback.
	NewtonBodySetTransformCallback( mBody, PhysicsObject::_newtonTransformCallback );

	// force and torque callback
	NewtonBodySetForceAndTorqueCallback( mBody, PhysicsObject::_newtonForceCallback );
}


void PhysicsObject::_updateVisualObjectMatrix( const float* matrix )
{
	// get the euler angles...
	dVector euler;
	dVector pos( matrix[12], matrix[13], matrix[14] );

	NewtonGetEulerAngle( matrix, &euler.m_x );

	// convert to degrees.
	euler = euler.Scale( 180.0f / 3.14159f );

	dbPositionObject( mObjNum, pos.m_x, pos.m_y, pos.m_z );
	dbRotateObject( mObjNum, euler.m_x, euler.m_y, euler.m_z );
}


// Newton transform callback:
void _cdecl PhysicsObject::_newtonTransformCallback(const NewtonBody* body, const float* matrix)
{
	PhysicsObject* me = (PhysicsObject*)NewtonBodyGetUserData(body);
	
	me->_updateVisualObjectMatrix( matrix );
}

// Newton force and torque calback
void _cdecl PhysicsObject::_newtonForceCallback(const NewtonBody* body )
{
	PhysicsObject* me = (PhysicsObject*)NewtonBodyGetUserData( body );

	dVector gravity(0.0f, _GAME_GRAVITY_ * me->getMass(), 0.0f);

	NewtonBodyAddForce( body, &gravity.m_x );
}



	


	



