
#include "SFX_Bomb.h"

SFX_Bomb::SFX_Bomb( BasicBall* bomb, dVector pos, int numParticles ) : SFXManager::SFXObject()
{
	// make the base particle piece.
	mBaseDebrisObjNum = Helpers::getOpenObjNum();
	dbMakeObjectTriangle( mBaseDebrisObjNum, -0.2f, 0.1f, 0.0f, 0.2f, -0.15f, 0.0f, -0.15f, -0.17f, 0.0f );
	dbColorObject( mBaseDebrisObjNum, dbRgb(200,0,0) );

	dbHideObject( mBaseDebrisObjNum );

	// make particles here.
	for (int i=0; i<numParticles; i++)
	{
		dVector vel = dVector((dbRnd(100)-50)/300.0f, (dbRnd(100)-50)/300.0f, (dbRnd(100)-50)/300.0f);
		DebrisParticle* part = new DebrisParticle(pos, vel, dbRnd(450.0), 0.6f, mBaseDebrisObjNum);
		mParticles.push_back( part );
	}

	// make fire particles.
	int fireImg = WorldInfo::getSingleton().mRootSmoke2ImgNum;

	for (int i=0; i<(numParticles/3); i++)
	{
		dVector vel = dVector((dbRnd(100)-50)/3.0f, (dbRnd(100)-50)/3.0f, (dbRnd(100)-50)/3.0f);
		SFX_Smoke::SmokeParticle* fire = new SFX_Smoke::SmokeParticle( pos, vel, dbRnd(200), 3.0f, 200.0f, 1.0f, fireImg );
		mFire.push_back( fire );
	}

	// now apply the bomb effect.
	WorldInfo::getSingleton().mBombBall = bomb;
	NewtonWorldForEachBodyDo( WorldInfo::getSingleton().mWorld, SFX_Bomb::newtonBombApply );
	WorldInfo::getSingleton().mBombBall = NULL;

	mDeleteDelay = 0;
}


SFX_Bomb::~SFX_Bomb()
{
	dbDeleteObject( mBaseDebrisObjNum );
	
}


void SFX_Bomb::_update(float timeStep)
{
	// update all bomb particles.
	DebrisParticleList toDelete;

	for (DebrisParticleListIterator it = mParticles.begin(); it != mParticles.end(); it++)
	{
		(*it)->update( timeStep );

		if ((*it)->mLife < 0.0f)
			toDelete.push_back( (*it) );
	}

	// and particles to delete?
	if (toDelete.size() > 0)
	{
		DebrisParticle* kill = toDelete.back();
		mParticles.remove( kill );

		toDelete.pop_back();
		delete kill;
	}

	//update all fire particles.
	SFX_Smoke::SmokeParticleList toDeleteFire;

	for (SFX_Smoke::SmokeParticleListIterator it = mFire.begin(); it != mFire.end(); it++)
	{
		(*it)->update( timeStep );

		if ((*it)->mLife < 0.0f)
			toDeleteFire.push_back( (*it) );
	}

	mDeleteDelay-= timeStep;

	if ((toDeleteFire.size() > 0) && (mDeleteDelay < 0.0f))
	{
		SFX_Smoke::SmokeParticle* kill = toDeleteFire.back();
		mFire.remove( kill );

		toDeleteFire.pop_back();
		delete kill;

		mDeleteDelay = 0.2;
	}
}


bool SFX_Bomb::_deleteme()
{
	return ((mParticles.size() == 0) && (mFire.size() == 0));
}


SFX_Bomb::DebrisParticle::DebrisParticle(dVector pos, dVector vel, float spin, float life, int objNum)
{
	// simple debris particle.
	mObjNum = Helpers::getOpenObjNum();
	mPos = pos;
	mVel = vel;
	mSpin = spin;
	mLife = life;

	dbInstanceObject( mObjNum, objNum );
	if (dbRnd(2) > 1)
		dbColorObject( mObjNum, dbRgb( 100+dbRnd(155), 0, 0 ) );
	else
		dbColorObject( mObjNum, dbRgb(200,200,200) );

	dbPositionObject( mObjNum, pos.m_x, pos.m_y, pos.m_z );
}

SFX_Bomb::DebrisParticle::~DebrisParticle()
{
	dbDeleteObject( mObjNum );
}

void SFX_Bomb::DebrisParticle::update( float timeStep )
{
	mLife -= timeStep;
	if (mLife < 0.0f)
		return;

	//gravity.
	dVector grav(0.0f,0.2f*timeStep,0.0f);
	mVel -= grav;

	mPos += mVel;

	dbRollObjectLeft( mObjNum, mSpin * timeStep );
	dbPositionObject( mObjNum, mPos.m_x, mPos.m_y, mPos.m_z );
	
	float scale = mLife * 300.0f;
	dbScaleObject( mObjNum, scale, scale, scale );
}

// bomb application.
void _cdecl SFX_Bomb::newtonBombApply( const NewtonBody* body )
{
	PhysicsObject* obj = (PhysicsObject*)NewtonBodyGetUserData( body );
	if (obj->getType() == BT_BALL)
	{
		BasicBall* ball = (BasicBall*)obj;

		dVector pos0 = WorldInfo::getSingleton().mBombBall->getPosition();
		dVector dist = pos0 - ball->getPosition();

		float length = Helpers::dVectorLength( dist );

		if (length <= 1.7f)
		{
			//delete this ball!
			WorldInfo::getSingleton().mToDelete[ ball->mObjNum ] = ball;
		}
		else
		{
			if ((length <= 4.0f) && (!ball->isPreviewBall()))
			{
				//apply an impulse to this guy.
				dVector pos = ball->getPosition();
				dVector vel = dist.Scale( (1.0f - length) * 0.25f );
				NewtonAddBodyImpulse( ball->mBody, &vel.m_x, &pos.m_x );
			}
		}
	}
}