/*
	ConnectMore project

	by Walaber

	CLASS: SoundManager - handles loading and playing all sounds / music.
*/

#ifndef _SOUNDMANAGER_INCLUDED_
#define _SOUNDMANAGER_INCLUDED_

#include "Prereqs.h"
#include "HelperFunctions.h"
#include <DarkSDK.h>


class SoundManager
{
public:
	// simple class wrapping basic sound functions
	class Sound
	{
	public:
		Sound( std::string filename );
		~Sound();
		
		void play();
		
		bool isPlaying();
		
	private:
		// dsdk sound number
		int mSoundNum;
	};

	class Music
	{
	public:
		Music( std::string filename );
		~Music();

		void play();
		void loopPlay();

		bool isPlaying();

		void stop();

	private:
		int mMusicNum;
	};
	
	
	//get singleton object reference
	static SoundManager& getSingleton();

	// add a sound to the system.
	Sound* addSound( std::string name, std::string filename );

	// get pointer to a sound
	Sound* getSound( std::string name );

	// remove a sound.
	void removeSound( std::string name );

	// simple play function
	void playSound( std::string name );


	// add music to the system.
	Music* addMusic( std::string name, std::string filename );

	// get pointer to music
	Music* getMusic( std::string name );

	// simple play function.
	void playMusic( std::string name );
	void loopMusic( std::string name );

	// remove music
	void removeMusic( std::string name );


private:
	SoundManager();
	~SoundManager();

	typedef std::map<std::string, Sound*> SoundMap;
	typedef std::map<std::string, Sound*>::iterator SoundMapIterator;

	SoundMap mSounds;


	typedef std::map<std::string, Music*> MusicMap;
	typedef std::map<std::string, Music*>::iterator MusicMapIterator;

	MusicMap mMusic;
};


#endif	// _SOUNDMANAGER_INCLUDED_
	
